From 63d6e6dfc35e6d82c4a64f37c408794c163becd4 Mon Sep 17 00:00:00 2001
From: thomascube <thomas@roundcube.net>
Date: Wed, 28 Sep 2011 15:16:41 -0400
Subject: [PATCH] Bump versions to 0.6 stable

---
 program/include/rcube_session.php |   88 ++++++++++++++++++++++++++++---------------
 1 files changed, 57 insertions(+), 31 deletions(-)

diff --git a/program/include/rcube_session.php b/program/include/rcube_session.php
index e4d9a87..01b9367 100644
--- a/program/include/rcube_session.php
+++ b/program/include/rcube_session.php
@@ -42,6 +42,7 @@
   private $prev;
   private $secret = '';
   private $ip_check = false;
+  private $logging = false;
   private $keep_alive = 0;
   private $memcache;
 
@@ -50,27 +51,21 @@
    */
   public function __construct($db, $config)
   {
-    $this->db = $db;
-    $this->start = microtime(true);
-    $this->ip = $_SERVER['REMOTE_ADDR'];
+    $this->db      = $db;
+    $this->start   = microtime(true);
+    $this->ip      = $_SERVER['REMOTE_ADDR'];
+    $this->logging = $config->get('log_session', false);
+    $this->mc_debug = $config->get('memcache_debug', false);
 
     $lifetime = $config->get('session_lifetime', 1) * 60;
     $this->set_lifetime($lifetime);
 
     // use memcache backend
     if ($config->get('session_storage', 'db') == 'memcache') {
-      $this->memcache = new Memcache;
-      $mc_available = 0;
-      foreach ($config->get('memcache_hosts', array()) as $host) {
-        list($host, $port) = explode(':', $host);
-        if (!$port) $port = 11211;
-        // add server and attempt to connect if not already done yet
-        if ($this->memcache->addServer($host, $port) && !$mc_available)
-          $mc_available += intval($this->memcache->connect($host, $port));
-      }
+      $this->memcache = rcmail::get_instance()->get_memcache();
 
       // set custom functions for PHP session management if memcache is available
-      if ($mc_available) {
+      if ($this->memcache) {
         session_set_save_handler(
           array($this, 'open'),
           array($this, 'close'),
@@ -131,10 +126,10 @@
   public function db_read($key)
   {
     $sql_result = $this->db->query(
-      "SELECT vars, ip, changed FROM ".get_table_name('session')." WHERE sess_id = ?",
-      $key);
+      "SELECT vars, ip, changed FROM ".get_table_name('session')
+      ." WHERE sess_id = ?", $key);
 
-    if ($sql_arr = $this->db->fetch_assoc($sql_result)) {
+    if ($sql_result && ($sql_arr = $this->db->fetch_assoc($sql_result))) {
       $this->changed = strtotime($sql_arr['changed']);
       $this->ip      = $sql_arr['ip'];
       $this->vars    = base64_decode($sql_arr['vars']);
@@ -161,10 +156,15 @@
     $ts = microtime(true);
     $now = $this->db->fromunixtime((int)$ts);
 
+    // no session row in DB (db_read() returns false)
+    if (!$this->key) {
+      $oldvars = false;
+    }
     // use internal data from read() for fast requests (up to 0.5 sec.)
-    if ($key == $this->key && (!$this->vars || $ts - $this->start < 0.5)) {
+    else if ($key == $this->key && (!$this->vars || $ts - $this->start < 0.5)) {
       $oldvars = $this->vars;
-    } else { // else read data again from DB
+    }
+    else { // else read data again from DB
       $oldvars = $this->db_read($key);
     }
 
@@ -198,8 +198,6 @@
    */
   private function _fixvars($vars, $oldvars)
   {
-    $ts = microtime(true);
-
     if ($oldvars !== false) {
       $a_oldvars = $this->unserialize($oldvars);
       if (is_array($a_oldvars)) {
@@ -261,8 +259,9 @@
    */
   public function mc_read($key)
   {
-    if ($value = $this->memcache->get($key)) {
-      $arr = unserialize($value);
+    $value = $this->memcache->get($key);
+    if ($this->mc_debug) write_log('memcache', "get($key): " . strlen($value));
+    if ($value && ($arr = unserialize($value))) {
       $this->changed = $arr['changed'];
       $this->ip      = $arr['ip'];
       $this->vars    = $arr['vars'];
@@ -287,16 +286,26 @@
   {
     $ts = microtime(true);
 
+    // no session data in cache (mc_read() returns false)
+    if (!$this->key)
+      $oldvars = false;
     // use internal data for fast requests (up to 0.5 sec.)
-    if ($key == $this->key && (!$this->vars || $ts - $this->start < 0.5))
+    else if ($key == $this->key && (!$this->vars || $ts - $this->start < 0.5))
       $oldvars = $this->vars;
     else // else read data again
       $oldvars = $this->mc_read($key);
 
     $newvars = $oldvars !== false ? $this->_fixvars($vars, $oldvars) : $vars;
     
-    if ($newvars !== $oldvars || $ts - $this->changed > $this->lifetime / 2)
-      return $this->memcache->set($key, serialize(array('changed' => time(), 'ip' => $this->ip, 'vars' => $newvars)), MEMCACHE_COMPRESSED, $this->lifetime);
+    if ($newvars !== $oldvars || $ts - $this->changed > $this->lifetime / 2) {
+      $value = serialize(array('changed' => time(), 'ip' => $this->ip, 'vars' => $newvars));
+      $ret = $this->memcache->set($key, $value, MEMCACHE_COMPRESSED, $this->lifetime);
+      if ($this->mc_debug) {
+        write_log('memcache', "set($key): " . strlen($value) . ": " . ($ret ? 'OK' : 'ERR'));
+        write_log('memcache', "... get($key): " . strlen($this->memcache->get($key)));
+      }
+      return $ret;
+    }
     
     return true;
   }
@@ -309,7 +318,9 @@
    */
   public function mc_destroy($key)
   {
-    return $this->memcache->delete($key);
+    $ret = $this->memcache->delete($key);
+    if ($this->mc_debug) write_log('memcache', "delete($key): " . ($ret ? 'OK' : 'ERR'));
+    return $ret;
   }
 
 
@@ -328,9 +339,9 @@
    */
   public function cleanup()
   {
-    // current compose information is stored in $_SESSION['compose'], move it to $_SESSION['compose_data']
-    if ($_SESSION['compose']) {
-      $_SESSION['compose_data'][$_SESSION['compose']['id']] = $_SESSION['compose'];
+    // current compose information is stored in $_SESSION['compose'], move it to $_SESSION['compose_data_<ID>']
+    if ($compose_id = $_SESSION['compose']['id']) {
+      $_SESSION['compose_data_'.$compose_id] = $_SESSION['compose'];
       $this->remove('compose');
     }
   }
@@ -573,12 +584,18 @@
     $this->cookie = $_COOKIE[$this->cookiename];
     $result = $this->ip_check ? $_SERVER['REMOTE_ADDR'] == $this->ip : true;
 
+    if (!$result)
+      $this->log("IP check failed for " . $this->key . "; expected " . $this->ip . "; got " . $_SERVER['REMOTE_ADDR']);
+
     if ($result && $this->_mkcookie($this->now) != $this->cookie) {
       // Check if using id from previous time slot
-      if ($this->_mkcookie($this->prev) == $this->cookie)
+      if ($this->_mkcookie($this->prev) == $this->cookie) {
         $this->set_auth_cookie();
-      else
+      }
+      else {
         $result = false;
+        $this->log("Session authentication failed for " . $this->key . "; invalid auth cookie sent");
+      }
     }
 
     return $result;
@@ -606,5 +623,14 @@
     $auth_string = "$this->key,$this->secret,$timeslot";
     return "S" . (function_exists('sha1') ? sha1($auth_string) : md5($auth_string));
   }
+  
+  /**
+   * 
+   */
+  function log($line)
+  {
+    if ($this->logging)
+      write_log('session', $line);
+  }
 
 }

--
Gitblit v1.9.1